/*	$Csoft	    */

/*
 * Copyright (c) 1999, 2000, 2001 CubeSoft Communications, Inc.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistribution of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistribution in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of CubeSoft Communications, nor the names of its
 *    contributors may be used to endorse or promote products derived from
 *    this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/mman.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <dirent.h>
#include <glib.h>

#include "techmlm-web.h"

static void 	hashmsg(struct tmweb *, struct tmhead *);
static void	archive_file(struct tmweb *, char *, char *, char *);
static void 	makewww(struct tmweb *, char *);
static void 	archive_dir(struct tmweb *, int, int);
static void 	web_archive(struct tmweb *);

static struct tmweb	*new_tmweb(void);
static void		free_tmweb(struct tmweb *);
static void 		quit(struct tmweb *);


static void 
hashmsg(struct tmweb *twi, struct tmhead *twh)
{
	if (!twh->depth)
		twi->headers = g_slist_append(twi->headers, twh);
	else
		twi->followups = g_slist_append(twi->followups, twh);
}

static void
archive_file(struct tmweb *twi, char *s, char *msgid, char *wf)
{
	struct tmhead *twh;
	struct stat sta;
	size_t fl;
	int f, nf;
	char *dat;

	if (!(f = open(s, O_RDONLY, 00600)))
		return;
	fl = lseek(f, 0, SEEK_END);

	if ((dat = mmap(0, fl, PROT_READ, MAP_PRIVATE, f, 0)) == MAP_FAILED) {
		perror("mmap");
		return;
	}
	twh = parse_header(twi, dat, fl, msgid);
	hashmsg(twi, twh);

	if (stat(wf, &sta) < 0) {
		/* Create html message file. */
		if (!(nf = open(wf, O_WRONLY | O_CREAT, 00600))) {
			close(f);
			return;
		}
		fchmod(nf, 00644);
		if (twi->maxmsglen && fl > twi->maxmsglen) {
			close(nf);
			close(f);
			fprintf(stderr, "msg %s size %i > %i\n", s, fl,
			    twi->maxmsglen);
			return;
		}
		layout_html(twi, nf, dat, fl, twh);
	}
	close(f);
}

static void 
makewww(struct tmweb *twi, char *dir)
{
	struct stat sta;

	if ((stat(twi->wwwdir, &sta) < 0) && (mkdir(twi->wwwdir, 00711) < 0))
		perror(twi->wwwdir);
	if ((stat(dir, &sta) < 0) && (mkdir(dir, 00711) < 0))
		perror(dir);
}

static void 
archive_dir(struct tmweb *twi, int mon, int year)
{
	char *s, *wwwd;
	struct dirent *di;
	DIR *d;

	s =  g_strdup_printf("%s/%i-%i/cases", twi->ardir, mon, year);
	wwwd = g_strdup_printf("%s/%i-%i", twi->wwwdir, mon, year);

	twi->mon = mon;
	twi->year = year;

	d = opendir(s);
	if (!d) {
		perror(s);
		return;
	}

	makewww(twi, wwwd);

	while ((di = readdir(d))) {
		char *ss = NULL, *wwwss = NULL, *msgid = NULL;
		if (!strcmp(di->d_name, ".") || !strcmp(di->d_name, ".."))
			continue;
		ss = g_strdup_printf("%s/%s", s, di->d_name);
		wwwss = g_strdup_printf("%s/%s.html", wwwd, di->d_name);
		msgid = strdup(di->d_name);
		archive_file(twi, ss, msgid, wwwss);
		free(ss);
	}
	closedir(d);
	free(wwwd);
	free(s);
}

static void 
web_archive(struct tmweb *twi)
{
	struct dirent *di;
	time_t mtime;
	struct tm *mtm;
	DIR *dirh;

	mtime = time(NULL);

	mtm = localtime(&mtime);
	mtm->tm_mon++;
	mtm->tm_year += 1900;

	dirh = opendir(twi->ardir);
	if (!dirh) {
		perror(twi->ardir);
		return;
	}
	while ((di = readdir(dirh))) {
		char **sl;

		if (!strcmp(di->d_name, ".") || !strcmp(di->d_name, ".."))
			continue;
		sl = g_strsplit(di->d_name, "-", 1);
		if ((atoi(sl[0]) == mtm->tm_mon) &&
		    (atoi(sl[1]) == mtm->tm_year)) {
			archive_dir(twi, atoi(sl[0]), atoi(sl[1]));
		}

		g_strfreev(sl);
	}
	closedir(dirh);

	techmlm_web_index(twi);
}

static struct tmweb *
new_tmweb(void)
{
	struct tmweb *twi;

	twi = malloc(sizeof(struct tmweb));
	if (twi == NULL) {
		perror("malloc");
		abort();
	}
	
	twi->ardir = NULL;
	twi->wwwdir = NULL;
	twi->baseurl = NULL;
	twi->maxmsglen = 0;
	twi->headers = NULL;
	twi->followups = NULL;

	return (twi);
}

static void
free_tmweb(struct tmweb *twi)
{
	if (twi->ardir)
		free(twi->ardir);
	if (twi->wwwdir)
		free(twi->wwwdir);
	if (twi->baseurl)
		free(twi->baseurl);
	if (twi->headers)
		g_slist_free(twi->headers);
	if (twi->followups)
		g_slist_free(twi->followups);
	free(twi);
}

static void 
quit(struct tmweb *twi)
{
	free_tmweb(twi);
	exit(0);
}

int 
main(int argc, char *argv[])
{
	struct tmweb *twi;
	int c;

	twi = new_tmweb();

	while ((c = getopt(argc, argv, "b:d:w:m:hv")) != EOF) {
		switch (c) {
		case 'b':
			twi->baseurl = strdup(optarg);
			break;
		case 'd':
			twi->ardir = strdup(optarg);
			break;
		case 'w':
			twi->wwwdir = strdup(optarg);
			break;
		case 'm':
			twi->maxmsglen = atoi(optarg);
			break;
		case 'v':
			fprintf(stderr, "techmlm %s\n", VERSION);
			quit(twi);
			break;
		case 'h':
		default:
			fprintf(stderr,
			    "Usage: %s [-v] [-m maxlen] [-b baseurl] "
			    "[-d ardir] [-w wwwdir]\n", argv[0]);
			quit(twi);
			break;
		}
	}

	if (!twi->ardir)
		twi->ardir = strdup(DEFAULT_ARDIR);
	if (!twi->wwwdir)
		twi->wwwdir = strdup(DEFAULT_WWWDIR);
	if (!twi->baseurl)
		twi->baseurl = strdup(DEFAULT_BASEURL);

	web_archive(twi);
	quit(twi);

	return (0);
}
